﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Operators;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Encodings.ConditionActionEncoding {
  [Item("ConditionActionSolutionAnalyzer", "")]
  [StorableClass]
  public abstract class XCSSolutionAnalyzer : SingleSuccessorOperator, IAnalyzer {
    public bool EnabledByDefault {
      get { return true; }
    }

    public ScopeTreeLookupParameter<IClassifier> ClassifierParameter {
      get { return (ScopeTreeLookupParameter<IClassifier>)Parameters["Classifier"]; }
    }
    public ScopeTreeLookupParameter<DoubleValue> PredictionParameter {
      get { return (ScopeTreeLookupParameter<DoubleValue>)Parameters["Prediction"]; }
    }
    public ScopeTreeLookupParameter<DoubleValue> ErrorParameter {
      get { return (ScopeTreeLookupParameter<DoubleValue>)Parameters["Error"]; }
    }
    public ScopeTreeLookupParameter<DoubleValue> FitnessParameter {
      get { return (ScopeTreeLookupParameter<DoubleValue>)Parameters["Fitness"]; }
    }
    public ScopeTreeLookupParameter<IntValue> ExperienceParameter {
      get { return (ScopeTreeLookupParameter<IntValue>)Parameters["Experience"]; }
    }
    public ScopeTreeLookupParameter<IntValue> TimestampParameter {
      get { return (ScopeTreeLookupParameter<IntValue>)Parameters["Timestamp"]; }
    }
    public ScopeTreeLookupParameter<DoubleValue> AverageActionSetSizeParameter {
      get { return (ScopeTreeLookupParameter<DoubleValue>)Parameters["AverageActionSetSize"]; }
    }
    public ScopeTreeLookupParameter<IntValue> NumerosityParameter {
      get { return (ScopeTreeLookupParameter<IntValue>)Parameters["Numerosity"]; }
    }
    public LookupParameter<IConditionActionProblemData> ProblemDataParameter {
      get { return (LookupParameter<IConditionActionProblemData>)Parameters["ProblemData"]; }
    }
    public ValueLookupParameter<ResultCollection> ResultsParameter {
      get { return (ValueLookupParameter<ResultCollection>)Parameters["Results"]; }
    }
    public ILookupParameter<IClassifierComparer> ClassifierComparerParameter {
      get { return (ILookupParameter<IClassifierComparer>)Parameters["ClassifierComparer"]; }
    }

    public ResultCollection Results { get { return ResultsParameter.ActualValue; } }

    [StorableConstructor]
    protected XCSSolutionAnalyzer(bool deserializing) : base(deserializing) { }
    protected XCSSolutionAnalyzer(XCSSolutionAnalyzer original, Cloner cloner) : base(original, cloner) { }
    public XCSSolutionAnalyzer()
      : base() {
      Parameters.Add(new ScopeTreeLookupParameter<IClassifier>("Classifier", ""));
      Parameters.Add(new ScopeTreeLookupParameter<DoubleValue>("Prediction", ""));
      Parameters.Add(new ScopeTreeLookupParameter<DoubleValue>("Error", ""));
      Parameters.Add(new ScopeTreeLookupParameter<DoubleValue>("Fitness", ""));
      Parameters.Add(new ScopeTreeLookupParameter<IntValue>("Experience", ""));
      Parameters.Add(new ScopeTreeLookupParameter<IntValue>("Timestamp", ""));
      Parameters.Add(new ScopeTreeLookupParameter<DoubleValue>("AverageActionSetSize", ""));
      Parameters.Add(new ScopeTreeLookupParameter<IntValue>("Numerosity", ""));
      Parameters.Add(new LookupParameter<IConditionActionProblemData>("ProblemData", ""));
      Parameters.Add(new ValueLookupParameter<ResultCollection>("Results", "The result collection where the solution should be stored."));
      Parameters.Add(new LookupParameter<IClassifierComparer>("ClassifierComparer"));
    }

    public override IOperation Apply() {
      ItemArray<IClassifier> classifiers = ClassifierParameter.ActualValue;
      ItemArray<DoubleValue> predictions = PredictionParameter.ActualValue;
      ItemArray<DoubleValue> errors = ErrorParameter.ActualValue;
      ItemArray<DoubleValue> fitnesses = FitnessParameter.ActualValue;
      ItemArray<IntValue> experiences = ExperienceParameter.ActualValue;
      ItemArray<IntValue> timestamps = TimestampParameter.ActualValue;
      ItemArray<DoubleValue> averageActionSetSizes = AverageActionSetSizeParameter.ActualValue;
      ItemArray<IntValue> numerosities = NumerosityParameter.ActualValue;
      IConditionActionProblemData problemData = ProblemDataParameter.ActualValue;

      ItemCollection<XCSClassifier> xcsClassifiers = new ItemCollection<XCSClassifier>();

      for (int i = 0; i < classifiers.Length; i++) {
        xcsClassifiers.Add(new XCSClassifier(classifiers[i], predictions[i], errors[i],
          fitnesses[i], experiences[i], timestamps[i], averageActionSetSizes[i], numerosities[i]));
      }

      XCSModel xcsModel = new XCSModel(xcsClassifiers);
      xcsModel.ClassifierComparer = ClassifierComparerParameter.ActualValue;
      UseCurrentXCSSolution(xcsModel.CreateConditionActionSolution(problemData));
      return base.Apply();
    }

    protected abstract void UseCurrentXCSSolution(IXCSSolution xcsSolution);
  }
}
